##***************************************************
## R source code for image manipulation
## Created by Myriam Vimond
## Updated 27 July 2016
##***************************************************

## read.grayImage()-------------------------------------
##     Reads graylevel image from a PNG file
## INPUT :
##     file = PNG file
##     nrow = range of row indices of the (sub)-image to load (ie a vector) or number or rows (ie a scalar)
##     ncol = range of column indices of the (sub)-image to load (ie a vector) or number or rows (ie a scalar)
## OUTPUT : 
##     a matrix objet of nonnegative real numbers
read.grayImage <- function(file,nrow=NULL, ncol=NULL){
  img <- readPNG(file)
  if(is.null(nrow)){nrow=1:NROW(img)}
  if(is.null(ncol)){ncol=1:NCOL(img)}
  if(length(nrow)==1){nrow=1:nrow}
  if(length(ncol)==1){ncol=1:ncol}
  img[nrow, ncol]
}

## write.grayImage()-------------------------------------
##     Writes a matrix in a PNG file
## INPUT :
##     x  = matrix
##     file = PNG file name
##     map = standardisation method to [0,1] of the matrix,
##           "direct": no standardization (pixels with intensity either <0 or >1 are set respectively to 0 or 1)
##           "scaled": standardization of pixel intensities to the interval (0,1)
## OUTPUT : no output


write.grayImage <- function(x, file, map="direct", ...){   
  if(map=="direct")
    {
     if(any(x>1) | any(x<0))
       {
        x[x>1] <- 1
        x[x<0] <- 0
        warning("the matrix has values which are <0 or >1") 
       }
   }
  if(map=="scaled")
    {
     x<-(x-min(x))/(max(x)-min(x))
     }
  writePNG(x, target=file)    
}

## plot.grayImage()-------------------------------------
##     Plots the matrix as a graylevel image
## INPUT :
##     x  = matrix
##     map = standardisation of the matrix,
##           "direct" no standardization
##           "scaled" standardization of the pixel intensity to the interval (0,1)
##     cData = number of graylevels, 
##           "bitmap" for two levels
##           "8bits" for 256 levels
##           "10bits" for 1024 levels
##           "12bits" for 4096 levels
## OUTPUT : no output
plot.grayImage <- function(x,cData="8bits",map="direct", ...){   # pourquoi ... dans la d?claration de la fonction ???
  graylevels<- switch(cData,
                      "bitmap" = 1, 
                      "8bits" = 255,
                      "10bits" = 1023, 
                      "12bits" = 4095)
  if(map=="scaled")
    {
     x<-(x-min(x))/(max(x)-min(x))
     }
  x<-(round(x*graylevels))
  image(t(x)[,nrow(x):1],col=gray(0:graylevels / graylevels),axes=FALSE,...)
}

## hist.grayImage()-------------------------------------
##     Plots the histogram of the graylevel image intensity
## INPUT :
##     x  = matrix
##     map = standardisation of the matrix,
##           "direct" no standardization
##           "scaled" standardization of the pixel intensity to the interval (0,1)
##     cData = number of graylevels, 
##           "bitmap" for two levels
##           "8bits" for 256 levels
##           "10bits" for 1024 levels
##           "12bits" for 4096 levels
## OUTPUT : no output
hist.grayImage<-function(x,cData="8bits",map="direct", ...)
{
  n1 <- NROW(x)
  n2 <- NCOL(x)
  n<-n1*n2
  graylevels<- switch(cData,
                      "bitmap" = 1, 
                      "8bits" = 255,
                      "10bits" = 1023, 
                      "12bits" = 4095)
  if(map=="scaled")
    {
     x<-(x-min(x))/(max(x)-min(x))
     }
  x<-(round(x*graylevels))/graylevels
  freq <- table(x)/n
  plot(freq,xlim=c(0,1),xaxt='n',main='',xlab='Intensity',ylab='Density')
  axis(1, at=seq(0,1,1/5),labels=seq(0,1,1/5))
}


## plot.grayIllumination()-------------------------------------
##    Plots the estimated illumination artifact as a level plot
## INPUT :
##     x  = matrix
##     levels = number of desired levels in the plot
## OUTPUT : no output
plot.grayIllumination <- function(x, levels = 15, ...){
  my.padding <- GrTheme()
  ## margin of the figure
  my.padding$layout.heights <- list(top.padding = 0,
                                    main = 0,
                                    main.key.padding = 0.5,
                                    key.top = 1,
                                    xlab.top = 1,
                                    key.axis.padding = 1,
                                    axis.top = 0,
                                    strip = 1,
                                    panel = 1,
                                    axis.panel =1,
                                    between = 1,
                                    axis.bottom = 0,
                                    axis.xlab.padding = 1,
                                    xlab = 1,
                                    xlab.key.padding = 0.5,
                                    key.bottom = 1,
                                    key.sub.padding = 1,
                                    sub =1,
                                    bottom.padding = 0)
  my.padding$layout.widths <- list(left.padding = 0,
                                   key.left = 1,
                                   key.lab.padding = 0.5,
                                   ylab = 1,
                                   ylab.axis.padding = 0.5,
                                   axis.left = 0,
                                   axis.panel =1,
                                   strip.left = 1,
                                   panel = 1,
                                   between = 1,
                                   axis.right = 0,
                                   axis.key.padding = 0.5,
                                   ylab.right = 1,
                                   key.right = 1,
                                   right.padding = 0)
  ## setting the parameters of the axis
  my.padding$axis.components$left <- list( tck=0, 
                                           pad1=0,
                                           pad2=1 )
  my.padding$axis.components$top <- list( tck=0, pad1=0 , pad2=1 )
  my.padding$axis.components$right <- list( tck=0, pad1=0 , pad2=1 )
  my.padding$axis.components$bottom <- list( tck=0, pad1=0 , pad2=0 )
  my.padding$axis.text <- list(alpha=0,
                               cex=0.8,
                               col='black',
                               font=1,
                               lineheight=1)
  n1<-nrow(x)
  n2<-ncol(x)
  ## setting the graylevels
  ml<-floor(min(log(x)))
  Ml<-ceiling(max(log(x)))
  k<-levels*(levels+1)/2
  my.at <- exp(c(-cumsum(abs(ml)*seq(1,levels,by=1)/k),seq(0,Ml,by=Ml/levels)))
  resl<-(t(x))[,seq(nrow(x), 1, by=-1)]
  
  levelplot(resl, 
            layers=1, 
            axes=FALSE, 
            margin=FALSE, 
            xlab= "", 
            ylab= "",
            par.settings=my.padding,
            at=my.at, 
            colorkey=list(TRUE,space="bottom"))
}
